# This script creates a five-node topology(see the document) and 
# three rsvp flows. All flows come from the same node(0) and have different 
# destination nodes(2,3,4). Also three trace files are created. Trace files are 
# created by logging the received bandwith in receiver node for every flow. By  
# examining the trace files one can see the effect on received bandwith when   
# reservations are made. 
set ns [new Simulator] 
 
$ns color 0 red 
$ns color 1 blue 
$ns color 2 green 
$ns color 3 yellow 
$ns color 46 purple 
 
 
# Create nodes 
set n0 [$ns node] 
set n1 [$ns node] 
set n2 [$ns node] 
set n3 [$ns node] 
set n4 [$ns node] 
 
 
# Open trace files and activate nam-trace 
set f0 [open band2.tr w] 
set f1 [open band3.tr w] 
set f2 [open band4.tr w] 
set nf [open example2.nam w] 
$ns namtrace-all $nf 
 
 
# Create rsvp links(see the document!) 
$ns duplex-rsvp-link $n0 $n1 1Mb 1s 0.9 200 5000 Param Null 
$ns duplex-rsvp-link $n1 $n2 1Mb 1s 0.9 200 5000 Param Null 
$ns duplex-rsvp-link $n1 $n3 1Mb 1s 0.9 200 5000 Param Null 
$ns duplex-rsvp-link $n1 $n4 1Mb 1s 0.9 200 5000 Param Null 
 
 
 
# Enable upcalls on all nodes 
Agent/RSVP set noisy_ 255 
 
 
# Create rsvp agents 
set rsvp4 [$n4 add-rsvp-agent] 
set rsvp3 [$n3 add-rsvp-agent] 
set rsvp2 [$n2 add-rsvp-agent] 
set rsvp1 [$n1 add-rsvp-agent] 
set rsvp0 [$n0 add-rsvp-agent] 
 
 
 
# Create three traffic sources with desired bitrates. 
# Note that for some reason the random variable have to be turned on. 
# Note also that because destination nodes are different for each flow, 
# also the values of "class_" variable have to be different.
set udp0 [new Agent/UDP] 
$ns attach-agent $n0 $udp0 
$udp0 set class_ 1 
set cbr0 [new Application/Traffic/CBR] 
$cbr0 set rate_ 0.5Mb 
$cbr0 set random_ 1 
$cbr0 attach-agent $udp0 
 
set udp1 [new Agent/UDP] 
$ns attach-agent $n0 $udp1 
$udp1 set class_ 2
set cbr1 [new Application/Traffic/CBR] 
$cbr1 set rate_ 0.5Mb 
$cbr1 set random_ 1 
$cbr1 attach-agent $udp1 
 
set udp2 [new Agent/UDP] 
$ns attach-agent $n0 $udp2 
$udp2 set class_ 3
set cbr2 [new Application/Traffic/CBR] 
$cbr2 set rate_ 0.5Mb 
$cbr2 set random_ 1 
$cbr2 attach-agent $udp2 
 
 
# Create sinks 
set sink2 [new Agent/LossMonitor] 
$ns attach-agent $n2 $sink2 
 
set sink3 [new Agent/LossMonitor] 
$ns attach-agent $n3 $sink3 
 
set sink4 [new Agent/LossMonitor] 
$ns attach-agent $n4 $sink4 
 
 
# Connect traffic sources to sinks 
$ns connect $udp0 $sink2 
$ns connect $udp1 $sink3 
$ns connect $udp2 $sink4 
 
 
# Create rsvp sessions for all rsvp agents(see the document). Now, because destination
# nodes are different, the values of "class_" also have to be different(1,2,3). 
set ses1 [$rsvp0 session $n2 1] 
set ses2 [$rsvp0 session $n3 2] 
set ses3 [$rsvp0 session $n4 3] 
 
 
# Start the procedure which logs the rate for the incoming flow on node 2,3 and# 4. 
$ns at 0.0 "record" 
 
# Send PATH-messages from each sender-node 
$ns at 1.0 "$rsvp0 sender $ses1 +500000 5000 32" 
$ns at 3.0 "$rsvp0 sender $ses2 +500000 5000 32" 
$ns at 5.0 "$rsvp0 sender $ses3 +500000 5000 32" 
 
 
# Reserve 0,8Mb/s for flow going to node 4. Notice that session id have 
# to be zero! 
$ns at 30.0 "$rsvp4 reserve 0 ff +800000 100000 $n0" 
 
 
# Start and stop traffic sources 
$ns at 10.0 "$cbr0 start" 
$ns at 50.0 "$cbr0 stop" 
$ns at 10.0 "$cbr1 start" 
$ns at 50.0 "$cbr1 stop" 
$ns at 10.0 "$cbr2 start" 
$ns at 50.0 "$cbr2 stop" 
 
$ns at 60.0 "finish" 
 
 
# This procedure logs the received bandwith for each flow. 
# For example the received bandwith in node two is logged 
# each second to trace file "band2.tr". 
proc record {} { 
	global sink2 sink3 sink4 ns f0 f1 f2 
	set time 1.0 
 
	set bw0 [$sink2 set bytes_] 
	set bw1 [$sink3 set bytes_] 
        set bw2 [$sink4 set bytes_] 
 
	set now [$ns now] 
 
	puts $f0 "$now [expr $bw0/$time*8]" 
	puts $f1 "$now [expr $bw1/$time*8]" 
	puts $f2 "$now [expr $bw2/$time*8]" 
 
	$sink2 set bytes_ 0 
	$sink3 set bytes_ 0 
	$sink4 set bytes_ 0 
 
	$ns at [expr $now+$time] "record" 
} 
 
 
proc finish {} { 
        global ns nf 
        $ns flush-trace 
        close $nf 
	puts "Done." 
        exit 0 
} 
 
$ns run 

